@@ -1,4 +1,3 @@
-import threading
 import anyio
 import logging
 import datetime
@@ -7,13 +6,14 @@ from pydantic import PrivateAttr
 from dagster import (
     AssetKey,
     AssetCheckResult,
+    AssetCheckSeverity,
     MaterializeResult,
     DataVersion,
     ConfigurableResource,
     InitResourceContext,
+    Failure
 )
 from sqlmesh import Context
-from sqlmesh.core.console import set_console
 from .translator import SQLMeshTranslator
 from .sqlmesh_asset_utils import (
     get_models_to_materialize,
@@ -22,9 +22,20 @@ from .sqlmesh_asset_utils import (
     get_model_partitions_from_plan,
     analyze_sqlmesh_crons_using_api,
     get_model_from_asset_key,
-    sanitize_metadata_for_dagster,
 )
-from .sqlmesh_event_console import SQLMeshEventCaptureConsole
+from .notifier_service import (
+    get_or_create_notifier,
+    register_notifier_in_context,
+    clear_notifier_state,
+)
+from .sqlmesh_asset_check_utils import (
+    extract_failed_audit_details,
+    deduplicate_asset_check_results,
+    serialize_audit_args,
+    create_failed_audit_check_result,
+    create_general_error_check_result,
+    convert_notifier_failures_to_asset_check_results,
+)
 from sqlmesh.utils.errors import (
     SQLMeshError,
     PlanError,
@@ -40,16 +51,18 @@ from sqlmesh.utils.errors import (
 import json
 
 
-class UpstreamAuditFailureError(Exception):
+
+class UpstreamAuditFailureError(Failure):
     """
     Custom exception for upstream audit failures that should be handled gracefully.
-    This exception should not trigger retries or be re-raised by the factory.
+    This exception is marked as non-retriable (allow_retries=False).
     """
 
-    pass
+    def __init__(self, description: str | None = None, metadata: dict | None = None):
+        super().__init__(description=description, metadata=metadata, allow_retries=False)
 
 
-def convert_unix_timestamp_to_readable(timestamp):
+def convert_unix_timestamp_to_readable(timestamp: float | int | None) -> str | None:
     """
     Converts a Unix timestamp to a readable date.
 
@@ -72,10 +85,6 @@ def convert_unix_timestamp_to_readable(timestamp):
         return str(timestamp)
 
 
-# Global lock for SQLMesh console singleton
-_console_lock = threading.Lock()
-
-
 class SQLMeshResource(ConfigurableResource):
     """
     Dagster resource for interacting with SQLMesh.
@@ -101,12 +110,7 @@ class SQLMeshResource(ConfigurableResource):
         if translator:
             self._translator_instance = translator
 
-        # Create SQLMesh console at initialization
-        self._console = self._get_or_create_console()
-
-        # Configure translator in console after creation
-        if hasattr(self, "_translator_instance") and self._translator_instance:
-            self._console._translator = self._translator_instance
+        # No console initialization (migrated to notifier-only)
 
     def __del__(self):
         pass  # Simplified cleanup
@@ -117,20 +121,9 @@ class SQLMeshResource(ConfigurableResource):
         return logging.getLogger(__name__)
 
     @classmethod
-    def _get_or_create_console(cls) -> "SQLMeshEventCaptureConsole":
-        """Creates or returns the singleton instance of the SQLMesh event console."""
-        # Initialize class variables lazily
-        if not hasattr(cls, "_console_instance"):
-            cls._console_instance = None
-
-        if cls._console_instance is None:
-            with _console_lock:
-                if cls._console_instance is None:  # Double-check pattern
-                    cls._console_instance = SQLMeshEventCaptureConsole(
-                        log_override=logging.getLogger(__name__),
-                    )
-                    set_console(cls._console_instance)
-        return cls._console_instance
+    def _get_or_create_notifier(cls):
+        """Deprecated internal accessor kept for backward compatibility; delegates to service."""
+        return get_or_create_notifier()
 
     @property
     def context(self) -> Context:
@@ -138,14 +131,12 @@ class SQLMeshResource(ConfigurableResource):
         Returns the SQLMesh context. Cached for performance.
         """
         if not hasattr(self, "_context_cache"):
-            # Configure custom console before creating context
-            console = self._get_or_create_console()
-            console._logger = self._logger  # Update logger
-
             self._context_cache = Context(
                 paths=self.project_dir,
                 gateway=self.gateway,
             )
+            # Register our notifier target at Context init via service (idempotent)
+            register_notifier_in_context(self._context_cache)
         return self._context_cache
 
     @property
@@ -165,9 +156,7 @@ class SQLMeshResource(ConfigurableResource):
         # Store Dagster logger in private attribute
         self._logger = context.log
 
-        # Configure console with Dagster logger
-        if hasattr(self, "_console") and self._console:
-            self._console._logger = self._logger
+        # No console configuration
 
     def get_models(self):
         """
@@ -187,11 +176,8 @@ class SQLMeshResource(ConfigurableResource):
         return analyze_sqlmesh_crons_using_api(self.context)
 
     def _serialize_audit_args(self, audit_args):
-        """Serialize audit arguments to JSON-compatible format"""
-        try:
-            return json.dumps(audit_args, default=str)
-        except Exception:
-            return "{}"
+        """Deprecated: use sqlmesh_asset_check_utils.serialize_audit_args"""
+        return serialize_audit_args(audit_args)
 
     def _deduplicate_asset_check_results(
         self, asset_check_results: list[AssetCheckResult]
@@ -215,18 +201,17 @@ class SQLMeshResource(ConfigurableResource):
                     grouped_results[key] = result
                     if self._logger:
                         self._logger.warning(
-                            f"⚠️ Conflicting audit results for {result.asset_key}.{result.check_name}: prioritizing failed result"
+                            f"Conflicting audit results for {result.asset_key}.{result.check_name}: prioritizing failed result"
                         )
 
         return list(grouped_results.values())
 
-    def materialize_assets(self, models, context=None):
-        """
-        Materializes specified SQLMesh assets with robust error handling.
-        """
+    def materialize_assets(self, models: list[Any], context: Any | None = None) -> Any:
+        """Materialize specified SQLMesh models with robust error handling."""
         model_names = [model.name for model in models]
-        self._prepare_console()
         try:
+            # Reset notifier state before starting a new materialization run
+            clear_notifier_state()
             plan = self._create_sqlmesh_plan(model_names)
             self._run_sqlmesh_plan(model_names)
             return plan
@@ -250,32 +235,26 @@ class SQLMeshResource(ConfigurableResource):
         except Exception as e:
             self._log_and_raise(f"Unexpected error: {e}")
 
-    def _prepare_console(self):
-        set_console(self._console)
-        self._console.clear_events()
-
-    def _create_sqlmesh_plan(self, model_names):
+    def _create_sqlmesh_plan(self, model_names: list[str]) -> Any:
         return self.context.plan(
             select_models=model_names,
             auto_apply=False,  # never apply the plan, we will juste need it for metadata collection
             no_prompts=True,
         )
 
-    def _run_sqlmesh_plan(self, model_names):
+    def _run_sqlmesh_plan(self, model_names: list[str]) -> None:
         self.context.run(
             environment=self.environment,
             select_models=model_names,
             execution_time=datetime.datetime.now(),
         )
 
-    def _log_and_raise(self, message):
+    def _log_and_raise(self, message: str) -> None:
         self._logger.error(message)
         raise
 
-    def materialize_assets_threaded(self, models, context=None):
-        """
-        Synchronous wrapper for Dagster that uses anyio.
-        """
+    def materialize_assets_threaded(self, models: list[Any], context: Any | None = None) -> Any:
+        """Synchronous wrapper for Dagster that uses anyio."""
 
         def run_materialization():
             try:
@@ -286,11 +265,8 @@ class SQLMeshResource(ConfigurableResource):
 
         return anyio.run(anyio.to_thread.run_sync, run_materialization)
 
-    def _extract_model_info(self, error) -> tuple[str, Any, Any]:
-        """
-        Extract model name, model object, and asset key from error.
-        Returns (model_name, model, asset_key)
-        """
+    def _extract_model_info(self, error: Any) -> tuple[str, Any | None, Any | None]:
+        """Extract (model_name, model, asset_key) from a SQLMesh error object."""
         model_name = "unknown"
         model = None
         asset_key = None
@@ -329,105 +305,53 @@ class SQLMeshResource(ConfigurableResource):
                     asset_key = self.translator.get_asset_key(model)
         except Exception as e:
             if self._logger:
-                self._logger.warning(f"⚠️ Error converting model name to asset key: {e}")
+                self._logger.warning(f"Error converting model name to asset key: {e}")
 
         return model_name, model, asset_key
 
     def _create_failed_audit_check_result(
         self, audit_error, model_name, asset_key
     ) -> AssetCheckResult | None:
-        """
-        Create AssetCheckResult for a failed audit.
-        Returns None if audit details cannot be extracted (just logs the error).
-        """
-        try:
-            audit_sql = audit_error.sql()
-            audit_name = getattr(audit_error, "audit_name", "unknown")
-            audit_args = getattr(audit_error, "audit_args", {})
-            audit_message = str(audit_error)
-            audit_blocking = getattr(audit_error, "blocking", False)
-            serialized_args = self._serialize_audit_args(audit_args)
-
-            # Log failed audit
-            if self._logger:
-                self._logger.warning(
-                    f"❌ AUDIT FAILED for model '{model_name}': {audit_name} - {audit_message}"
-                )
-
-            # Sanitize metadata for Dagster 1.11.4 compatibility
-            metadata = {
-                "sqlmesh_model_name": model_name,
-                "audit_query": audit_sql,
-                "audit_blocking": audit_blocking,
-                "audit_message": audit_message,
-                "audit_args": serialized_args,
-                "error_type": "audit_failure",
-            }
-            sanitized_metadata = sanitize_metadata_for_dagster(metadata)
-            
-            return AssetCheckResult(
-                passed=False,  # Failed audit
-                asset_key=asset_key,
-                check_name=audit_name,
-                metadata=sanitized_metadata,
-            )
-        except Exception as audit_e:
-            if self._logger:
-                self._logger.warning(
-                    f"⚠️ Failed to extract audit details for {model_name}: {audit_e}"
-                )
-            return None
+        """Deprecated: use sqlmesh_asset_check_utils.create_failed_audit_check_result"""
+        return create_failed_audit_check_result(
+            audit_error=audit_error,
+            model_name=model_name,
+            asset_key=asset_key,
+            logger=self._logger,
+        )
 
     def _create_general_error_check_result(
         self, error, model_name, asset_key, error_type: str, message: str
     ) -> AssetCheckResult:
-        """
-        Create AssetCheckResult for general errors (non-audit).
-        """
-        # Log general error
-        if self._logger:
-            self._logger.warning(
-                f"❌ MODEL ERROR for model '{model_name}': {error_type} - {message}"
-            )
-
-        # Sanitize metadata for Dagster 1.11.4 compatibility
-        metadata = {
-            "sqlmesh_model_name": model_name,
-            "audit_query": "N/A",
-            "audit_blocking": False,
-            "audit_message": message,
-            "audit_args": {},
-            "error_type": error_type,
-        }
-        sanitized_metadata = sanitize_metadata_for_dagster(metadata)
-        
-        return AssetCheckResult(
-            passed=False,
+        """Deprecated: use sqlmesh_asset_check_utils.create_general_error_check_result"""
+        return create_general_error_check_result(
+            error=error,
+            model_name=model_name,
             asset_key=asset_key,
-            check_name="model_execution_error",
-            metadata=sanitized_metadata,
+            error_type=error_type,
+            message=message,
+            logger=self._logger,
         )
 
-    def _process_failed_models_events(self) -> list[AssetCheckResult]:
+    def _process_notifier_audit_failures(self) -> list[AssetCheckResult]:
         """
-        Process failed models events and convert them to AssetCheckResult.
+        Convert notifier-captured audit failures into AssetCheckResult.
+        Honors blocking flag to set severity and downstream blocking.
         """
-        failed_models_events = self._console.get_failed_models_events()
-        asset_check_results = []
+        try:
+            from .notifier_service import get_audit_failures
+            failures = get_audit_failures()
+        except Exception:
+            failures = []
 
-        for event in failed_models_events:
-            errors = event.get("errors", [])
-            for error in errors:
-                try:
-                    model_name, model, asset_key = self._extract_model_info(error)
-                    self._process_single_error(
-                        error, model_name, asset_key, asset_check_results
-                    )
-                except Exception as e:
-                    self._log_failed_error_processing(e)
-        return asset_check_results
+        return convert_notifier_failures_to_asset_check_results(
+            context=self.context,
+            translator=self.translator,
+            failures=failures,
+            logger=self._logger,
+        )
 
-    def _process_single_error(self, error, model_name, asset_key, asset_check_results):
+    def _process_single_error(self, error: Any, model_name: str, asset_key: Any, asset_check_results: list[AssetCheckResult]) -> None:
         # Process audit errors if present
         if hasattr(error, "__cause__") and error.__cause__:
             if isinstance(error.__cause__, NodeAuditsErrors):
@@ -450,8 +374,8 @@ class SQLMeshResource(ConfigurableResource):
             asset_check_results.append(general_result)
 
     def _process_audit_errors(
-        self, audits_errors, model_name, asset_key, asset_check_results
-    ):
+        self, audits_errors: Any, model_name: str, asset_key: Any, asset_check_results: list[AssetCheckResult]
+    ) -> None:
         for audit_error in audits_errors.errors:
             audit_result = self._create_failed_audit_check_result(
                 audit_error, model_name, asset_key
@@ -459,9 +383,9 @@ class SQLMeshResource(ConfigurableResource):
             if audit_result is not None:
                 asset_check_results.append(audit_result)
 
-    def _log_failed_error_processing(self, exception):
+    def _log_failed_error_processing(self, exception: Exception) -> None:
         if self._logger:
-            self._logger.warning(f"⚠️ Failed to process error: {exception}")
+            self._logger.warning(f"Failed to process error: {exception}")
 
     def _get_failed_blocking_checks(
         self, asset_check_results: list[AssetCheckResult]
@@ -555,48 +479,27 @@ class SQLMeshResource(ConfigurableResource):
             self.context, self.translator, selected_asset_keys
         )
 
-        # Process all audit results (successful and failed)
-        successful_audit_results = []
-        audit_results = self._console.get_audit_results()
-        for audit_result in audit_results:
-            audit_details = audit_result["audit_details"]
-            asset_key = audit_result["asset_key"]
+        # No audit successes gathered from console (handled via factory by default)
+        successful_audit_results: list[AssetCheckResult] = []
 
-            # Successful audits are always passed=True
-            passed = True
+        # Legacy console path removed
+        failed_models_results: list[AssetCheckResult] = []
 
-            # Serialize audit arguments to JSON-compatible format
-            serialized_args = self._serialize_audit_args(audit_details["arguments"])
+        # Process audits captured via notifier (blocking and non-blocking)
+        notifier_audit_results = self._process_notifier_audit_failures()
 
-            # Sanitize metadata for Dagster 1.11.4 compatibility
-            metadata = {
-                "sqlmesh_model_name": audit_result[
-                    "model_name"
-                ],  # ← SQLMesh model name
-                "audit_query": audit_details["sql"],
-                "audit_blocking": audit_details["blocking"],
-                "audit_dialect": getattr(audit_details, "dialect", "unknown"),
-                "audit_args": serialized_args,
-                "error_type": "audit_success",
-            }
-            sanitized_metadata = sanitize_metadata_for_dagster(metadata)
-            
-            successful_audit_results.append(
-                AssetCheckResult(
-                    passed=passed,
-                    asset_key=asset_key,
-                    check_name=audit_details["name"],
-                    metadata=sanitized_metadata,
-                )
-            )
-
-        # Process failed models/audits
-        failed_models_results = self._process_failed_models_events()
+        # No console warning path
+        non_blocking_warning_results: list[AssetCheckResult] = []
 
         # Combine and deduplicate all AssetCheckResult events
-        all_asset_check_results = successful_audit_results + failed_models_results
-        deduplicated_results = self._deduplicate_asset_check_results(
-            all_asset_check_results
+        all_asset_check_results = (
+            successful_audit_results
+            + failed_models_results
+            + non_blocking_warning_results
+            + notifier_audit_results
+        )
+        deduplicated_results = deduplicate_asset_check_results(
+            all_asset_check_results, logger=self._logger
         )
 
         # Get failed blocking checks and affected downstream assets
@@ -618,7 +521,7 @@ class SQLMeshResource(ConfigurableResource):
                 # Skip affected downstream assets - don't yield MaterializeResult
                 if self._logger:
                     self._logger.warning(
-                        f"⏭️ Skipping materialization of {asset_key} due to upstream failures: "
+                        f"Skipping materialization of {asset_key} due to upstream failures: "
                         f"{[str(key) for key in failed_asset_keys]}"
                     )
                 # Raise custom exception that will be handled gracefully by factory
@@ -655,20 +558,16 @@ class SQLMeshResource(ConfigurableResource):
                         # Ensure partition metadata is compatible with Dagster 1.11.4
                         metadata["dagster-sqlmesh/partitions"] = partition_metadata
 
-                    # Sanitize metadata for Dagster 1.11.4 compatibility
-                    sanitized_metadata = sanitize_metadata_for_dagster(metadata)
-
                     # Get check results for this asset
                     check_results = asset_check_results_by_key.get(asset_key, [])
 
                     yield MaterializeResult(
                         asset_key=asset_key,
-                        metadata=sanitized_metadata,
+                        metadata=metadata,
                         data_version=DataVersion(str(snapshot_version))
                         if snapshot_version
                         else None,
                         check_results=check_results,
                     )
 
-        # Clean console events after emitting all AssetCheckResult
-        self._console.clear_events()
+        # End notifier-only path
